## ---------------------------
##
## Run leave-out analysis
##
## ---------------------------

if(!dir.exists(file.path(base_dir, "leaveout"))) dir.create(file.path(base_dir, "leaveout"))

if(exists("optim_param")) {
  init_lo_param <- optim_param
} else {
  init_lo_param <- tmb_param
}

## Message

time_lo_start <- Sys.time()
message("Leave out started ", time_lo_start)
cat("Sequentially dropping")
lapply(leaveout_re, function(k) grep(k, tmb_param$switch, value = TRUE)) %>% setNames(leaveout_re)

## Leave out parameter list

leaveout_pars <- lapply(leaveout_re, function(k) {
  init_lo_param %>%
    g3_init_guess(k, 0, NA, NA, 0) %>% 
    mutate(value = ifelse(is.infinite(unlist(value)), 0, value))
}) %>% setNames(leaveout_re)

save(leaveout_pars, file = file.path(base_dir, "leaveout", "leaveout_pars_in.Rdata"), compress = "xz")

# Optimisation ####

lo_param <- parallel::mclapply(seq_along(leaveout_pars), function(i) {
  
  out <- g3_optim(model = tmb_model,
                  params = leaveout_pars[[i]],
                  use_parscale = TRUE,
                  method = 'BFGS',
                  control = list(maxit = maxit #, reltol = 1e-9
                  ),
                  print_status = TRUE,
                  print_id = names(leaveout_pars)[i]
  )
  
  return(out)
  
}, mc.cores = ifelse(ncores > length(leaveout_pars), length(leaveout_pars), ncores)
) %>% setNames(leaveout_re)

lo_param <- lapply(lo_param, function(k) if(inherits(k, "try-error")) NULL else k) # Clean crashes not to crash the entire script

time_lo_end <- Sys.time()
time_lo <- round(as.numeric(time_lo_end - time_lo_start, units = "mins"), 1)
message("Optimization finished ", time_lo_end, " after ", time_lo, " min")

## Write the times to a file
if(!file.exists(file.path(base_dir, "session/run_times.txt"))) {
  info_file <- file(file.path(base_dir, "session/run_times.txt"))
  close(info_file)
}

cat(
  c("Leave out:\n",
    "   started ", as.character(time_lo_start), "\n",
    "   finished ", as.character(time_lo_end), "\n",
    "   time ", time_lo, " min", "\n",
    "   element names: ", paste(names(sapply(lo_param, function(k) attributes(k)$summary$method)), collapse = ", "), "\n",
    "   method: ", paste(sapply(lo_param, function(k) attributes(k)$summary$method), collapse = ", "), "\n",
    "   convergence: ", paste(sapply(lo_param, function(k) attributes(k)$summary$convergence), collapse = ", "), "\n",
    "   iterations: ", paste(sapply(lo_param, function(k) attributes(k)$summary$gd_calls), collapse = ", "), "\n",
    "   score: ", paste(sapply(lo_param, function(k) {
      out <- attributes(k)$summary$score
      if(is.null(out)) NULL else round(out,1)
    })
    , collapse = ", "), "\n\n"),
  file = file.path(base_dir, "session/run_times.txt"), sep = "")

### Save the model parameters

save(lo_param, file = file.path(base_dir, "leaveout/Optimized leaveout parameters.rda"), compress = "xz")

## Fit

lo_fit <- parallel::mclapply(seq_along(lo_param), function(i) {
  gadgetutils:::echo_message(paste0("Generating fit: ", i, "/", length(lo_param)))
  if(is.null(lo_param[[i]])) return(NULL)
  if(inherits(lo_param[[i]], "try-error")) return(NULL)
  try(g3_fit(model = tmb_model, params = lo_param[[i]]))
}, mc.cores = ifelse(ncores > length(leaveout_pars), length(leaveout_pars), ncores)
) %>% setNames(leaveout_re)

lo_fit <- Filter(
  Negate(is.null),
  lapply(lo_fit, function(k) if(inherits(k, "try-error")) NULL else k)
)

save(lo_fit, file = file.path(base_dir, "leaveout/Leaveout fit.rda"), compress = "xz")

## Plots

ggsave(file.path(base_dir, "figures/Leaveout_analysis.png"),
       plot = gadgetplots::plot_leaveout(lo_fit),
       width = pagewidth, height = pagewidth*0.7, units = "mm")
