## ---------------------------
##
## Script name: Catches
##
## Purpose of script: Load catch data from MFDB
##
## ---------------------------



## Source or list custom functions used within the script

source("R/figure_functions.R")

## ---------------------------

## Read data

catch_nor <- readr::read_rds("data/Catches from IMR database.rds") %>%
  filter(nation == "NOR") %>%
  dplyr::select(-date, -gear_cat, -nation, -sub_area, -gear_cat, -gear_id) %>%
  rename(areacell = main_area, weight_total = mass) %>%
  mutate(sampling_type = "LND", institute = "NOR")

load("data/Russian catches.rda")

catch_rus <- catch_rus %>%
  mutate(month = as.integer(recode(step, "1" = "1", "2" = "4", "3" = "7", "4" = "10"))) %>%
  mutate(areacell = 14,
         gear = ifelse(gear == "trawl", "BottomTrawls", "Longlines"),
         weight_total = value*1e3
  ) %>%
  dplyr::select(year, month, areacell, gear, weight_total) %>%
  bind_rows(
    readxl::read_excel("data/GHL Russia catches 2022.xlsx", "catches", skip = 2) %>%
      rename_with(tolower) %>%
      filter(quarter != "Total") %>%
      mutate(month = as.integer(recode(quarter, "1" = "1", "2" = "4", "3" = "7", "4" = "10"))) %>%
      mutate(areacell = 14,
             gear = ifelse(gear == "trawl", "BottomTrawls", "Longlines"),
             weight_total = total*1e3
      ) %>%
      dplyr::select(year, month, areacell, gear, weight_total)
  )

catch_hist <- readr::read_rds("data/AFWG table 8-7 historic catches.rds")
catch_int <- readr::read_rds("data/International_catches_2006-2022.rds")

## ---------------------------

# TrawlNor ####

TrawlNor_catches <- g3_data(
  x = catch_nor %>%
    filter(gear %in% c("BottomTrawls", "DanishSeines", "OtherTrawls", "PelagicTrawls",
                       "Seines", "ShrimpTrawls"),
           sampling_type == "LND",
           institute == "NOR"),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/TrawlNor_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(TrawlNor_catches))
dev.off()

# OtherNor ####

OtherNor_catches <- g3_data(
  x = catch_nor %>%
    filter(!gear %in% c("BottomTrawls", "DanishSeines", "OtherTrawls", "PelagicTrawls",
                        "Seines", "ShrimpTrawls"),
           sampling_type == "LND",
           institute == "NOR"),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/OtherNor_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(OtherNor_catches))
dev.off()

# TrawlRus ####

TrawlRus <- g3_data(
  x = catch_rus %>%
    filter(gear %in% c("BottomTrawls")),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )


OtherRus <- g3_data(
  x = catch_rus %>%
    filter(gear %in% c("Longlines")),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )


## Split factor
split_fac <- full_join(
  TrawlRus %>% rename("TrawlRus" = "total_weight"),
  OtherRus %>% rename("OtherRus" = "total_weight"),
  by = c("year", "step", "area")) %>%
  filter(year < 2001, year > 1990) %>%
  mutate(prop = OtherRus/(TrawlRus+OtherRus)) %>%
  pull(prop) %>%
  mean()

# HistRus ####
HistRus <- g3_data(
  catch_hist %>%
    rename(year = Year) %>%
    filter(year < 1991) %>%
    mutate(weight_total = 1e3*Russia) %>%
    select(-Norway, -Russia, -Others, -Total) %>%
    na.omit() %>%
    mutate(month = 6, areacell = 26, .before = "weight_total") ,
  params = list(year = model_params$year_range,
                timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step),
         TrawlRus = (1-split_fac)*total_weight,
         OtherRus = split_fac*total_weight) %>%
  dplyr::select(year, step, area, TrawlRus, OtherRus) %>%
  na.omit()

# TrawlRus ####

TrawlRus_catches <- bind_rows(
  HistRus %>% dplyr::select(-OtherRus) %>% rename("total_weight" = "TrawlRus"),
  TrawlRus %>% filter(year > 1990)
)

# OtherRus ####

OtherRus_catches <- bind_rows(
  HistRus %>% dplyr::select(-TrawlRus) %>% rename("total_weight" = "OtherRus"),
  OtherRus %>% filter(year > 1990)
)

rm(HistRus, TrawlRus, OtherRus, split_fac)

png(file.path(base_dir, "figures/TrawlRus_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(TrawlRus_catches))
dev.off()

png(file.path(base_dir, "figures/OtherRus_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(OtherRus_catches))
dev.off()

# International catches ####

Internat_catches <- g3_data(
  x =
    catch_hist %>%
    rename(year = Year) %>%
    filter(year >= 1980) %>%
    mutate(weight_total = 1e3*Others) %>%
    select(-Norway, -Russia, -Others, -Total) %>%
    bind_rows(
      catch_int %>%
        filter(Year == "2022", Area == "All") %>%
        rename(year = Year) %>%
        mutate(
          year = as.integer(year),
          weight_total = 1e3*sum(across(where(is.numeric)))) %>%
        dplyr::select(year, weight_total) %>%
        ungroup()
    ) %>%
    mutate(month = 6, areacell = 26),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun),
  method = "weight_total"
) %>%
  rename(total_weight = weight_total) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/Internat_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(Internat_catches))
dev.off()

# Survey dummy catches ####

if(!exists("EggaN_ldist")) source("2-3 catch distribution.R")

EggaN_catches <- structure(
  data.frame(
    year = unique(EggaN_ldist$year), step = 1, area = 1, total_weight = 1),
  area_group = mfdb_group(`1` = 1)) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/EggaN_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(EggaN_catches))
dev.off()

EggaS_catches <- structure(
  data.frame(
    year = unique(EggaS_ldist$year), step = 1, area = 1, total_weight = 1),
  area_group = mfdb_group(`1` = 1)) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/EggaS_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(EggaS_catches))
dev.off()

EcoS_catches <- structure(
  data.frame(
    year = unique(EcoS_ldist$year), step = 1, area = 1, total_weight = 1),
  area_group = mfdb_group(`1` = 1)) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/EcoS_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(EcoS_catches))
dev.off()

WinterS_catches <- structure(
  data.frame(
    year = unique(WinterS_ldist$year), step = 1, area = 1, total_weight = 1),
  area_group = mfdb_group(`1` = 1)) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/WinterS_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(WinterS_catches))
dev.off()

RussianS_catches <- structure(
  data.frame(
    year = unique(RussianS_ldist$year), step = 1, area = 1, total_weight = 1),
  area_group = mfdb_group(`1` = 1)) %>%
  mutate(area = 1, # These hacks are required due to a bug in gadget3
         step = as.numeric(step)
  )

png(file.path(base_dir, "figures/RussianS_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(RussianS_catches))
dev.off()

## All catches

png(file.path(base_dir, "figures/All_catches.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(plot.catches(list("TrawlNor" = TrawlNor_catches,
                        "OtherNor" = OtherNor_catches,
                        "TrawlRus" = TrawlRus_catches,
                        "OtherRus" = OtherRus_catches,
                        "Internat" = Internat_catches)) +
        # ggplot2::guides(fill=ggplot2::guide_legend(nrow=2,byrow=TRUE)) +
        ggplot2::theme(legend.position = "bottom"))
dev.off()

# Save

save(TrawlNor_catches, OtherNor_catches, TrawlRus_catches, OtherRus_catches, Internat_catches, EggaN_catches, EggaS_catches, EcoS_catches, WinterS_catches, RussianS_catches, file = file.path(base_dir, "data/Catches to Gadget.rda"))

