## ---------------------------
##
## Run jitter analysis by slightly varying optimised model parameters
##
## ---------------------------


if(exists("iter_param")) {
  init_jitter_param <- iter_param
} else if(exists("optim_param")) {
  init_jitter_param <- optim_param
} else {
  init_jitter_param <- tmb_param
}

jitpar_out <- gadgetutils::g3_jitter(
  gd = base_dir,
  outdir = "jitter",
  model = tmb_model,
  params = init_jitter_param,
  njits = 20,
  control = list(maxit = maxit #, reltol = 1e-9
  ),
  ncores = 1)

jitpar_list <- lapply(seq_along(jitpar_out), function(i) {
  if(is.null(jitpar_out[[i]])) return(NULL)
  if(inherits(jitpar_out[[i]], "try-error")) return(NULL)
  out <- jitpar_out[[i]][jitpar_out[[i]]$optimise, c("switch", "value")]
  rownames(out) <- 1:nrow(out)
  names(out)[names(out) == "value"] <- paste0("value_run",i)
  out
}) %>%
  purrr::discard(is.null) %>%
  purrr::reduce(dplyr::full_join, by = "switch") %>%
  dplyr::rowwise() %>%
  dplyr::mutate(
    mean = mean(dplyr::c_across(dplyr::starts_with("value")), na.rm = TRUE),
    sd = sd(dplyr::c_across(dplyr::starts_with("value")), na.rm = TRUE),
    cv = abs(sd/mean))

jitpar_list %>%
  write.g3.file(file.path(base_dir, "jitter"), 'jitter.param.comparison')

p <- ggplot2::ggplot(
  data = jitpar_list,
  ggplot2::aes(.data$switch,.data$cv,label=.data$switch)) +
  ggplot2::geom_point() +
  ggplot2::coord_flip() +
  ggplot2::labs(
    x = "",
    y = paste0("Parameter CV over ",
               length(grep("^value", names(jitpar_list))), " jitter runs")
  ) +
  ggplot2::theme_bw(base_size = 8)

ggsave(file.path(base_dir, "figures/Jitter_parameter_CV.png"),
       plot = p, width = pagewidth, height = pagewidth*1.5, units = "mm")

jitter_fit <- parallel::mclapply(seq_along(jitpar_out), function(i) {
  message(paste0(i, "/", length(jitpar_out)))
  if(is.null(jitpar_out[[i]])) return(NULL)
  if(inherits(jitpar_out[[i]], "try-error")) return(NULL)
  try(g3_fit(model = tmb_model, params = jitpar_out[[i]]))
},
mc.cores = 1)

jitter_fit <- Filter(
  Negate(is.null),
  lapply(jitter_fit, function(k) if(inherits(k, "try-error")) NULL else k)
)

save(jitter_fit, file = file.path(base_dir, "jitter", 'jitter_fit.rda'), compress = "xz")

gadgetplots:::bind_fit_components(jitter_fit, 'score') %>%
  na.omit() %>%
  summarise(
    n_jitter = n(),
    mean = mean(nll),
    sd = sd(nll),
    cv = sd/mean) %>%
  write.g3.file(file.path(base_dir, "jitter"), 'jitter.nll.summary')

ggsave(file.path(base_dir, "figures/Jitter_model_biomass.png"),
       plot = gadgetplots::plot_jitter(jitter_fit),
       width = pagewidth, height = pagewidth*0.7, units = "mm")

