## -----------------------------------------------------------------------------
##
## Projection functions
##
## -----------------------------------------------------------------------------

proj_stock_actions2 <- function(num_project_years,
                                mat, imm_F, imm_M,
                                comp_id = c('species', 'sex')) {

  out_stocks <- list(imm_F, imm_M)

  ## Setup up a dummy stock
  dummy_stock <-
    gadget3::g3_stock(c(species = mat$name_parts[['species']], 'dummy'),
                      lengthgroups = seq(min(gadget3::g3_stock_def(imm_F, 'minlen')),
                                         max(head(gadget3::g3_stock_def(imm_F, 'maxlen'),-1)),
                                         min(gadget3::g3_stock_def(imm_F, 'dl')))) %>%
    gadget3::g3s_livesonareas(areas[c('1')]) %>%
    gadget3::g3s_age(minage = 0, maxage = gadget3::g3_stock_def(imm_F, 'minage')-1)


  ## Ageing:
  ## To ensure the numbers match, the spawned population should transition when recruitment occurs
  dummy_actions <-
    list(
      gadget3::g3a_age(dummy_stock,
                       output_stocks = out_stocks,
                       run_f = ~cur_step == 1,
                       run_at = 1,
                       transition_at = 8)
    )

  spawning_actions <-
    list(
      gadget3::g3a_spawn(
        stock = mat,
        output_stocks = list(dummy_stock),
        recruitment_f =
          g3a_spawn_recruitment_hockeystick(
            r0 = gadget3:::f_substitute(
              ~scale * g3_param_table('project_rec',
                                      expand.grid(
                                        cur_year = seq(end_year - minage,
                                                       end_year + py)), ifmissing = 0),
              list(py = num_project_years,
                   minage = gadget3::g3_stock_def(imm_F, 'minage'),
                   scale = 1e4)),
            blim = g3_parameterized('blim')),
        proportion_f = 1,
        mean_f = g3_parameterized('recl', by_stock = 'species'),
        stddev_f = g3_parameterized('rec.sd', by_stock = 'species'),
        alpha_f = g3_parameterized('walpha', by_stock = comp_id),
        beta_f = g3_parameterized('wbeta', by_stock = comp_id),
        #run_f = ~cur_year_projection && cur_step == 1)
        run_f = gadget3:::f_substitute(~cur_year > (end_year - (minage + 1)) && cur_step == 1,
                                       list(end_year = local(end_year),
                                            minage = gadget3::g3_stock_def(imm_F, 'minage'))))

    )

  return(c(dummy_actions, spawning_actions))

}

runfun <- function(adfun, pars, hr_len = 45){

  ## Schedule
  schedule <-
    expand.grid(year = min(year_range):(max(year_range)+num_project_years),
                step = 1:length(defaults$timestep)) %>%
    arrange_all() %>%
    mutate(time = as.character(1:nrow(.)))

  ## Retrieve reports
  res <- adfun$report(gadget3::g3_tmb_par(pars))

  ## Recruitment
  rec <- res[names(res)[grepl('^proj_(.+)_spawnednum$', names(res))]] %>%
    map(.f = function(x) as.data.frame.table(x, stringsAsFactors = FALSE)[,c('time','Freq')]) %>%
    bind_rows(.id = 'comp') %>%
    mutate(stock = gsub('proj_(.+.)_dummy__spawnednum$', '\\1', comp)) %>%
    group_by(time) %>%
    summarise(rec = sum(Freq), .groups = 'drop') %>%
    left_join(schedule, by = 'time') %>%
    select(year, rec) %>%
    mutate(year = year)

  ## Catches
  tmp <-
    res[names(res)[grepl('^proj_(.+)__predby_(.+)_proj$', names(res))]] %>%
    map(.f = function(x) as.data.frame.table(x, stringsAsFactors = FALSE)) %>%
    bind_rows(.id = 'comp') %>%
    gadgetutils:::split_length() %>%
    mutate(length = (ifelse(is.infinite(upper), lower + 1, upper) + lower )/2,)

  catch <-
    tmp %>%
    group_by(time) %>%
    summarise(catch = sum(Freq, na.rm = TRUE), .groups = 'drop')
  # %>%
  #   left_join(
  #     tmp %>%
  #       filter(length >= hr_len) %>%
  #       group_by(time) %>%
  #       summarise(hr_catch = sum(Freq, na.rm = TRUE), .groups = 'drop')
  #     , c('time'))
  #

   WL <-
    ## Stock weight and numbers
    res[names(res)[grepl('^proj_(.+)_(imm|mat)__(wgt$|num$|spawningnum$)', names(res))]] %>%
    map(.f = function(x) as.data.frame.table(x, stringsAsFactors = FALSE)) %>%
    bind_rows(.id = 'comp') %>%
    mutate(stock = gsub('^proj_(.+)__(wgt$|num$|spawningnum$)', '\\1', comp),
           var = gsub('^proj_(.+)_(.+)_(.+)__(.+)', '\\4', comp)) %>%
    select(-comp) %>%
    pivot_wider(names_from = var, values_from = Freq) %>%
    select(-area) %>%
    mutate(biomass = wgt*num,
           biomass_that_spawns = wgt * spawningnum)

  out <-
    WL %>%
    gadgetutils:::split_length() %>%
    mutate(length = (ifelse(is.infinite(upper), lower + 1, upper) + lower )/2,) %>%
    group_by(time) %>%
    summarise(total_biomass = sum(biomass, na.rm = TRUE),
              hr_biomass = sum(biomass[length >= hr_len], na.rm = TRUE),
              .groups = 'drop') %>%
    left_join(
      WL %>%
        filter(stock == 'ghl_female_mat') %>%
        group_by(time) %>%
        summarise(ssb = sum(biomass, na.rm = TRUE),
                  ssb_that_spawns = sum(biomass_that_spawns, na.rm = TRUE), .groups = 'drop')
      , by = 'time') %>%
    left_join(catch, by = c('time')) %>%
    mutate(hrbar = catch/hr_biomass) %>%
    mutate(hr = catch/total_biomass) %>%
    replace_na(list(hrbar = 0)) %>%
    left_join(schedule, by = 'time') %>%
    left_join(rec, by = 'year') %>%
    select(year, step, catch, hrbar, ssb, ssb_that_spawns, rec) %>%
    rename(hr = hrbar) %>%
    arrange(year) %>%
    filter(year >= local(start_year))


  return(out)

}

g3p_project_report2 <- function (actions,
                                abundance_run_at = 1,
                                run_at = 11) {
  c(
    gadget3::g3a_report_history(
      actions = actions,
      var_re = c('__num$', '__wgt$'),
      out_prefix = "proj_",
      run_f = ~TRUE,
      run_at = abundance_run_at),
    gadget3::g3a_report_history(
      actions = actions,
      var_re = c('__renewalnum$', '__spawnednum$', '__predby_', '__spawningnum$'),
      out_prefix = 'proj_',
      run_f = ~TRUE,
      run_at = run_at),
    NULL
  )
}
